#!perl -w
#  I Calendar
#    change #!/usr/bin/perl or so, as you need

#
# Class OBJECT
#
{
  package OBJECT;
  use strict;
  use Date::Calc qw(Add_Delta_Days Day_of_Week);
  
  sub today()
  {
    return((localtime)[5]+1900,(localtime)[4]+1,(localtime)[3]);
  }

  sub dateformat()
  {
    my $class = shift;
    my ($year,$month,$date) = @_;
    $year  =~ s/.*(\d\d)$/$1/;
    $month = ($month < 10)? '0' . $month : $month;
    $date  = ($date  < 10)? '0' . $date  : $date;
    return ( $year . ':' . $month . ':' . $date);
  }

  sub datenext()
  {
    my $class = shift;
    my ($year,$month,$date) = @_;
    return(Add_Delta_Days($year,$month,$date,1));
  }

  sub dateprev()
  {
    my $class = shift;
    my ($year,$month,$date) = @_;
    return(Add_Delta_Days($year,$month,$date,-1));
  }

  sub dayofweek()
  {
    my $class = shift;
    my ($year,$month,$date) = @_;
    return ('MON','TUE','WED','THR','FRI','SAT','SUN')
       [Day_of_Week($year,$month,$date)-1];
  }
} ### END PART

#
# Class ICALENDAR
#
{
  package ICALENDAR;
  use strict;
  use base qw(OBJECT);
  use URI::Escape;
  use Storable;
  use Fcntl ':flock';

  my $users = 'users';
  my $decks = 'decks';
  my $notes = 'notes';
  my $lock  = 'lock';
  my $FH_LOCK;

  sub new()
  {
    my $class = shift;
    my ($now,$key);
    my $self  = {
      DECKS => undef,
      USERS => undef,
      NOTES => undef,
      PROTO => undef,
      TYPE  => undef,
      COOKY => undef,
      INPUT => undef
    };
    bless $self,$class;
    open LOCK,">$lock";
    $FH_LOCK = \*LOCK;
    flock($FH_LOCK,LOCK_EX);
    $self->{USERS} = retrieve($users);
    $self->{DECKS} = retrieve($decks) if (-e $decks);
    $self->{NOTES} = retrieve($notes) if (-e $notes);
    $now  = time;
    for $key (keys %{$self->{DECKS}}){
      if(($now - $self->{DECKS}{$key}{LAST}) >= 300){
        delete $self->{DECKS}{$key};
      }
    }
    return $self;
  }

  sub exit()
  {
    my $self = shift;
    store $self->{USERS}, $users;
    store $self->{DECKS}, $decks;
    store $self->{NOTES}, $notes if defined $self->{NOTES};
    flock($FH_LOCK,LOCK_UN);
  }

  sub parse()
  {
    my $self = shift;
    my $buffer;
    my (@args,%inputs,%ret);
    my ($pair,@pairs,$key,$value);

    $buffer = $self->debug() if (-t);
    if(defined $ENV{CONTENT_LENGTH} && $ENV{CONTENT_LENGTH} >= 200){
      return undef;
    }

    if ($ENV{REQUEST_METHOD} eq 'GET'){
      $_ = $ENV{'QUERY_STRING'};
      @args = split /\?/;
      $self->{PROTO} = 'GET';
      $self->{TYPE} = shift @args;
      $self->{COOKY} = shift @args;
    }elsif($ENV{REQUEST_METHOD} eq 'POST'){
      $_ = $ENV{'QUERY_STRING'};
      @args = split /\?/;
      read(STDIN,$buffer,$ENV{CONTENT_LENGTH}) unless (-t);
      @pairs = split(/&/,$buffer);
      for $pair (@pairs){
        ($key,$value) = split(/=/,$pair);
        $value =~ tr/+/ /;
        $value = uri_unescape($value);
        $inputs{$key} = $value;
      }
      $self->{PROTO} = 'POST';
      $self->{TYPE}  = shift @args;
      $self->{COOKY} = shift @args;
      $self->{INPUT} = \%inputs,
    }else{
      return undef;
    }
    return 1;
  }

  sub debug()
  {
    my $self = shift;
    my ($key,$value,$buffer);
    print("DEBUG::REQUEST_METHOD> ");
    $_ = <>;
    chomp;
    $ENV{REQUEST_METHOD} = $_;
    print("DEBUG::QUERY_STRING  > ");
    $_ = <>;
    chomp;
    $ENV{QUERY_STRING} = $_;
    if($ENV{REQUEST_METHOD} eq 'POST'){
      print("DEBUG::POST_DATA e.g. name=value, end with new line.\n");
      while(($_ = <>) !~ /^\s+$/){
        chomp;
        ($key,$value) = split /=/;
        $value =~ tr/ /+/;
        $value = uri_escape($value);
        $buffer .= $key . '=' . $value . '&';
      }
      chop $buffer;
      $ENV{CONTENT_LENGTH} = length($buffer);
    }
    return $buffer;
  }

  sub decksIpush()
  {
    my $self = shift;
    my $deck = shift;
    $self->{DECKS}{$deck->{COOKY}} = $deck;
  }

  sub decksIpull()
  {
    my $self = shift;
    my $deck = shift;
    delete $self->{DECKS}{$deck->{COOKY}};
  }

  sub decksIsearch()
  {
    my $self = shift;
    return $self->{DECKS}{$self->{COOKY}};
  }

  sub usersIcheck()
  {
    my $self = shift;
    if ((defined $self->{USERS}{$self->{INPUT}{USER}}) && 
               ($self->{USERS}{$self->{INPUT}{USER}} eq $self->{INPUT}{PASS})){
      return $self->{INPUT}{USER};
    }else{
      return undef;
    }
  }
 
  sub usersIchange()
  {
    my $self = shift;
    if ((defined $self->{USERS}{$self->{INPUT}{USER}}) && 
               ($self->{USERS}{$self->{INPUT}{USER}} eq $self->{INPUT}{PASS})){
      $self->{USERS}{$self->{INPUT}{USER}} = $self->{INPUT}{NEW};
      return $self->{INPUT}{USER};
    }else{
      return undef;
    }
  }

  sub notesIsearch()
  {
    my $self = shift;
    my($user,$date,$type) = @_;
    if(defined $self->{NOTES}{$user}{$date}{$type}){
      return $self->{NOTES}{$user}{$date}{$type};
    }else{
      return '';
    }
  }

  sub  notesIpush()
  {
    my $self = shift;
    my ($user,$date,$type,$text) = @_;
    $self->{NOTES}{$user}{$date}{$type} = $text;
  }

  sub notesIpull()
  {
    my $self = shift;
    my ($user,$date) = @_;
    delete $self->{NOTES}{$user}{$date}
      if defined ($self->{NOTES}{$user}{$date});
  }

  sub cookyIexist()
  {
    my $self  = shift;
    my $cooky = shift;
    if (defined $self->{DECKS}{$cooky}){
      return 1;
    }else{
      return undef;
    }
  }

} ### END ICALENDAR

#
# Class DECK
#
{
  package DECK;
  use strict;
  use base qw(OBJECT);

  sub new()
  {
    my $class = shift;
    my $cooky = shift;
    my $self  = {
      USER    => undef,
      COOKY   => undef,
      COMMAND => undef,
      YEAR    => undef,
      MONTH   => undef,
      DATE    => undef,
      LAST    => undef,
    };
    bless $self, $class;
    $self->{COOKY} = $cooky;
    ($self->{YEAR},$self->{MONTH},$self->{DATE}) = $class->SUPER::today;
    return $self;
  }

  sub exec()
  {
    my $self    = shift;
    my $command = shift;
    my $ical    = shift;
    $self->{COMMAND} = $command;
    $self->{LAST}    = time;
    $command->exec($self,$ical);
  }

} ### END DECK

#
# Class COMMAND
#
{
  package COMMAND;
  use strict;
  use base qw(OBJECT);

  sub new()
  {
    my $class = shift;
    my $card  = shift;
    my $self  = {
      CARD  => $card,
    };
    bless $self,$class;
    return $self;
  }

  sub exec()
  {
    my $self = shift;
    my $deck  = shift;
    my $ical  = shift;
    if($self->action($deck,$ical)){
      $self->{CARD}->print($deck,$ical);
    }else{
      CARD::ERROR->print($deck,$ical);
    }
  }
} ### END COMMAND

#
# Class COMMAND::GETDECK
#
{
  package COMMAND::GETDECK;
  use strict;
  use base qw(COMMAND);

  sub action(){1}

} ### END COMMAND::GETDECK

#
# Class COMMAND::LOGIN
#
{
  package COMMAND::LOGIN;
  use strict;
  use base qw(COMMAND);

  sub action()
  {
    my $self = shift;
    my $deck = shift;
    my $ical = shift;
    my $name;
    if ($name = $ical->usersIcheck){
      $deck->{USER} = $name;
      return 1;
    }else{
      $deck->{COMMAND} = undef;
      return undef;
    }
  }
} ### END COMMAND::LOGIN

#
# Class COMMAND::MENU
#
{
  package COMMAND::MENU;
  use strict;
  use base qw(COMMAND);

  sub action()
  {
    my $self = shift;
    my $deck = shift;
    my $ical = shift;
    my $class = ref $self;
    if($ical->{TYPE} eq 'NEXT'){
      ($deck->{YEAR},$deck->{MONTH},$deck->{DATE}) = 
        $class->SUPER::datenext($deck->{YEAR},$deck->{MONTH},$deck->{DATE});
    }elsif($ical->{TYPE} eq 'PREV'){
      ($deck->{YEAR},$deck->{MONTH},$deck->{DATE}) =
        $class->SUPER::dateprev($deck->{YEAR},$deck->{MONTH},$deck->{DATE});
    }elsif($ical->{TYPE} eq 'TODAY'){
      ($deck->{YEAR},$deck->{MONTH},$deck->{DATE}) = $class->SUPER::today;
    }elsif($ical->{TYPE} eq 'CLEAR'){
      $ical->notesIpull($deck->{USER},
        $class->SUPER::dateformat($deck->{YEAR},
                                  $deck->{MONTH},$deck->{DATE}));
    }
    return 1;
  }
} ### END COMMAND::MENU

#
# Class COMMAND::VIEW
#
{
  package COMMAND::VIEW;
  use strict;
  use base qw(COMMAND);

  sub action()
  {
    my $self = shift;
    my $deck = shift;
    my $ical = shift;
    return 1;
  }

} ### END COMMAND::VIEW

#
# Class COMMAND::CHPW
#
{
  package COMMAND::CHPW;
  use strict;
  use base qw(COMMAND);

  sub action()
  {
    my $self = shift;
    my $deck = shift;
    my $ical = shift;
    if(($ical->{TYPE} eq 'NOP') || ($ical->usersIchange())){
      return 1;
    }else{
      return undef;
    }
  }
}
#
# Class COMMAND::EDIT
#
{
  package COMMAND::EDIT;
  use strict;
  use base qw(COMMAND);

  sub action()
  {
    my $self = shift;
    my $deck = shift;
    my $ical = shift;
    my $class = ref $self;
    unless($ical->{TYPE} eq 'CANCEL'){
      $ical->notesIpush(
        $deck->{USER},
        $class->SUPER::dateformat($deck->{YEAR},$deck->{MONTH},$deck->{DATE}),
        $ical->{TYPE},
        $ical->{INPUT}{HEAD});
      $ical->notesIpush(
        $deck->{USER},
        $class->SUPER::dateformat($deck->{YEAR},$deck->{MONTH},$deck->{DATE}),
        $ical->{TYPE}.'BODY',
        $ical->{INPUT}{BODY});
    }
    return 1;
  }
} ### END COMMAND::EDIT

#
# Class COMMAND::LOGOUT
#
{
  package COMMAND::LOGOUT;
  use strict;
  use base qw(COMMAND);

  sub action()
  {
    my $self = shift;
    my $deck = shift;
    my $ical  = shift;
    $ical->decksIpull($deck);
    return 1;
  }
} ### END COMMAND::LOGOUT

#
# Class CARD
#
{
  package CARD;
  use strict;
  use base qw(OBJECT);

  sub print()
  {
    my $class = shift;
    my ($deck,$ical) = @_;
    print "Content-type:text/html\r\n\r\n",$class->fetch($deck,$ical);
  }

} ### END CARD

#
# Class CARD::GETDECK
#
{
  package CARD::GETDECK;
  use strict;
  use base qw(CARD);

  sub _template()
  { 
    <<'END'
<html>
<title>iMode Calendar</title>
<body>
  <div align = "center">
    &#63862;Calendar <!-- i -->
  </div>
  <form method = "post" action = "/cgi-bin/ical.pl?LOGIN?%%KEY%%">
    &#63879;User:<br> <!-- 1 -->
    <input type = "text" name = "USER" size = "10"
      maxlength = "10" accesskey = "1"><br>
    &#63880;Pass:<br> <!-- 2 -->
    <input type = "password" name = "PASS" size = "10"
      maxlength = "10" accesskey = "2">
    <p>
    <input type = "submit" value = "_SEND" accesskey = "3">
    (&#63881;);<br>
    <input type =  "reset" value = "RESET" accesskey = "4">
    (&#63882;);
  </form>
  <a href = "/cgi-bin/ical.pl?LOGOUT?%%KEY%%" accesskey = "0">&#63888;
  LOGOUT</a>
  <br>
  <div align = "right">Ver. 0.01</div>
</body>
</html>
END
  }

  sub fetch()
  {
    my $class = shift;
    my ($deck,$ical) = @_;
    my $card = _template;
    $card =~ s/%%KEY%%/$deck->{COOKY}/g;
    return $card;
  }
} ### END CARD::GETDECK

#
# Class CARD::MENU
#
{
  package CARD::MENU;
  use strict;
  use base qw(CARD);

  sub _template ()
  {
    <<'END'
<html>
<body>
  &#63862;Cal%%THISDAY%%
  <br>
  User_:%%USER%%
  <br>
  <a href = "/cgi-bin/ical.pl?PLAN?%%KEY%%" accesskey = "1">&#63879;
  PLAN:</a>%%PLAN%%
  <br>
  <a href= "/cgi-bin/ical.pl?AM?%%KEY%%" accesskey = "2">&#63880;
  _A_M:</a>%%AM%%
  <br>
  <a href = "/cgi-bin/ical.pl?PM?%%KEY%%" accesskey = "3">&#63881;
  _P_M:</a>%%PM%%
  <br>
  <a href = "/cgi-bin/ical.pl?EVE?%%KEY%%" accesskey = "4">&#63882;
  _EVE:</a>%%EVE%%
  <br>
  <hr>
  <a href = "/cgi-bin/ical.pl?CLEAR?%%KEY%%" accesskey = "5">&#63883;
  CLEAR</a>
  <br>
  <a href = "/cgi-bin/ical.pl?CHPW?%%KEY%%" accesskey = "6">&#63884;
  Change Passwd</a>
  <br>
  <a href = "/cgi-bin/ical.pl?PREV?%%KEY%%" accesskey = "7">&#63885;
  Prev Day</a>
  <br>
  <a href = "/cgi-bin/ical.pl?TODAY?%%KEY%%" accesskey = "8">&#63886;
  Today</a>
  <br>
  <a href = "/cgi-bin/ical.pl?NEXT?%%KEY%%" accesskey = "9">&#63887;
  Next Day</a>
  <br>
  <a href = "/cgi-bin/ical.pl?LOGOUT?%%KEY%%" accesskey = "0">&#63888;
  LOGOUT</a>
  <br>
</body>
</html>
END
  }

  sub fetch()
  {
    my $class = shift;
    my ($deck,$ical) = @_;
    my $date = $class->SUPER::dateformat
                 ($deck->{YEAR},$deck->{MONTH},$deck->{DATE});
    my $card = _template;
    $card =~ s/%%KEY%%/$deck->{COOKY}/g;
    $card =~ s/%%USER%%/$deck->{USER}/;
    $card =~ s/%%PLAN%%/$ical->notesIsearch($deck->{USER},$date,'EDITPLAN')/e;
    $card =~ s/%%AM%%/$ical->notesIsearch($deck->{USER},$date,'EDITAM')/e;
    $card =~ s/%%PM%%/$ical->notesIsearch($deck->{USER},$date,'EDITPM')/e;
    $card =~ s/%%EVE%%/$ical->notesIsearch($deck->{USER},$date,'EDITEVE')/e;
    $date .= $class->SUPER::dayofweek
              ($deck->{YEAR},$deck->{MONTH},$deck->{DATE});
    $card =~ s/%%THISDAY%%/$date/;
    return $card;
  }

} ### END CARD::MENU

#
# Class CARD::VIEW
#
{
  package CARD::VIEW;
  use strict;
  use base qw(CARD);

  sub _template ()
  {
    <<'END'
<html>
<body>
  &#63862;Cal%%THISDAY%%
  <br>
  Head___%%HEAD%%
  <br>
  Body_(%%TYPE%%)
  <br>
  %%BODY%%
  <br>
  <a href = "/cgi-bin/ical.pl?EDIT%%TYPE%%?%%KEY%%" accesskey = "5">&#63883;
  EDIT</a>
  <br>
  <a href = "/cgi-bin/ical.pl?BACK?%%KEY%%" accesskey = "6">&#63884;
  BACK</a>
  <br>
  <a href = "/cgi-bin/ical.pl?LOGOUT?%%KEY%%" accesskey = "0">&#63888;
  LOGOUT</a>
</body>
</html>
END
  }

  sub fetch()
  {
    my $class = shift;
    my ($deck,$ical) = @_;
    my $card = _template;
    my $date = $class->SUPER::dateformat
                ($deck->{YEAR},$deck->{MONTH},$deck->{DATE});
    $card =~ s{%%HEAD%%}
      {$ical->notesIsearch($deck->{USER},$date,'EDIT'.$ical->{TYPE})}e;
    $card =~ s/%%TYPE%%/$ical->{TYPE}/g;
    $card =~ s{%%BODY%%}
      {$ical->notesIsearch($deck->{USER},$date,'EDIT'.$ical->{TYPE}.'BODY')}e;
    $card =~ s/%%KEY%%/$deck->{COOKY}/g;
    $date .= $class->SUPER::dayofweek
              ($deck->{YEAR},$deck->{MONTH},$deck->{DATE});
    $card =~ s/%%THISDAY%%/$date/;
    return $card;
  }

} ### END CARD::VIEW

#
# Class CARD::CHPW
#
{
  package CARD::CHPW;
  use strict;
  use base qw(CARD);

  sub _template ()
  {
    <<'END'
<html>
<body>
  User&#63868;%%USER%%
  <form method = "post" action = "/cgi-bin/ical.pl?CHPW?%%KEY%%">
    OLD&#63869;(&#63879;)<br>
    <input type = "hidden" name = "USER" value = "%%USER%%">
    <input type = "password" name = "PASS" size = "10"
      maxlength = "10" accesskey = "1"><br>
    NEW&#63869;(&#63880;)<br>
    <input type = "password" name = "NEW" size = "10"
      maxlength = "10" accesskey = "2"><br>
    <input type = "submit" value = "_SEND" accesskey = "3">
      (&#63881;)<br>
    <input type = "reset" value = "RESET" accesskey = "4">
      (&#63882;)<br>
  </form>
  <a href = "/cgi-bin/ical.pl?NOP?%%KEY%%" accesskey = "6">&#63884;
  BACK</a>
  <br>
  <a href = "/cgi-bin/ical.pl?LOGOUT?%%KEY%%" accesskey = "0">&#63888;
  LOGOUT</a>
  <br>
</body>
</html>
END
  }

  sub fetch()
  {
  my $class = shift;
  my ($deck,$ical) = @_;
  my $card  = _template;
  $card =~ s/%%USER%%/$deck->{USER}/g;
  $card =~ s/%%KEY%%/$deck->{COOKY}/g;
  return $card;
  }

} ### END CARD::CHPW

#
# Class CARD::EDIT
#
{
  package CARD::EDIT;
  use strict;
  use base qw(CARD);

  sub _template ()
  {
    <<'END'
<html>
<body>
  &#63862;Cal%%THISDAY%%
  <br>
  <form method = "post" action = "/cgi-bin/ical.pl?%%TYPE%%?%%KEY%%">
    &#63879;Head:<br> <!-- 1 -->
    <input type = "text" name = "HEAD" size = "8"
      maxlength =  "8" accesskey = "1" value = "%%HEAD%%">
    <br>
    &#63880;Body:<br> <!--- 2 -->
    <input type = "text" name = "BODY" size = "48"
      maxlength = "48" accesskey = "2" value = %%BODY%%>
    <p>
    <input type = "submit" value = "_SEND" accesskey = "5">
    (&#63883;)
    <br>
    <input type = "reset" value = "RESET" accesskey = "8">
    (&#63886;)
  </form>
  <hr>
  <a href = "/cgi-bin/ical.pl?CANCEL?%%KEY%%" accesskey = "6">
    &#63884; <!-- 6 -->
  BACK</a>
  <br>
  <a href = "/cgi-bin/ical.pl?LOGOUT?%%KEY%%" accesskey ="0">&#63888;<!-- 0 -->
  LOGOUT</a>
  <br>
</body>
</html>
END
  }

  sub fetch()
  {
    my $class = shift;
    my ($deck,$ical) = @_;
    my $card  = _template;
    my $date  = $class->SUPER::dateformat
                  ($deck->{YEAR},$deck->{MONTH},$deck->{DATE});
    $card =~ s/%%KEY%%/$deck->{COOKY}/g;
    $card =~ s/%%TYPE%%/$ical->{TYPE}/g;
    $card =~ s{%%HEAD%%}
             {$ical->notesIsearch($deck->{USER},$date,$ical->{TYPE})}e;
    $card =~ s{%%BODY%%}
             {$ical->notesIsearch($deck->{USER},$date,$ical->{TYPE}.'BODY')}e;
    $date .= $class->SUPER::dayofweek
              ($deck->{YEAR},$deck->{MONTH},$deck->{DATE});
    $card =~ s/%%THISDAY%%/$date/;
    return $card;
  }

} ### END CARD::EDIT

#
# Class CARD::ERROR
#
{
  package CARD::ERROR;
  use strict;
  use base qw(CARD);

  sub _template ()
  {
    <<'END'
<html>
<body>
  <p align = "center">
    &#63862;Calendar <!-- i -->
  </p>
  <a href= "/cgi-bin/ical.pl?%%GOBACK%%" accesskey = "1">&#63879;<!-- 1 -->
    GoBack</a><br>
  &#63904; <!-- Good idea -->
  Error found. Push &#63879;to go back
  <p>
  <div align="right">
  (C)2000.;
  <br>K.Daiba
  </div>
</body>
</html>
END
  }

  sub fetch()
  {
    my $self = shift;
    my ($deck,$ical) = @_;
    my $card = _template;
    if(defined $deck->{COMMAND}){
      my $type = ref($deck->{COMMAND});
      $type =~ s/COMMAND:://;
      $card =~ s/%%GOBACK%%/$type . '?' . $deck->{COOKY}/e;
    }else{
      $card =~ s/%%GOBACK%%/GETDECK/;
    }
    return $card;
  }

} ### END CARD::ERROR

#
# Class CARD::LOGOUT
#
{
  package CARD::LOGOUT;
  use strict;
  use base qw(CARD);

  sub fetch()
  {
    <<'END'
<html>
<body>
  <p align = "center">
    &#63862;Calendar <!-- i -->
  </p>
    &#63890; Good BYE!
  <br>
  <a href = "/index.html" accesskey = "1">&#63879;
  Try again</a>
  <p>
  <div align="right">
  (C)2000.;
  <br>
  K.Daiba
  </div>
</body>
</html>
END
  }
}

#
# main package
#

use strict;
my ($ical,%table,$key,$cName,$command,$card,$deck,$cooky);

srand(time() ^ ($$ + ($$ << 15)));

map {$table{$_->[0]} = {COMMAND => $_->[1],CARD => $_->[2]}}
  map[m{(\S+)\s+(\S+)\s+(\S+)}],<<'END' =~ /(.+)/g;
GET_GETDECK   COMMAND::GETDECK CARD::GETDECK
POST_LOGIN    COMMAND::LOGIN   CARD::MENU
GET_TODAY     COMMAND::MENU    CARD::MENU
GET_PREV      COMMAND::MENU    CARD::MENU
GET_NEXT      COMMAND::MENU    CARD::MENU
GET_CLEAR     COMMAND::MENU    CARD::MENU
GET_CHPW      COMMAND::MENU    CARD::CHPW
GET_NOP       COMMAND::CHPW    CARD::MENU
POST_CHPW     COMMAND::CHPW    CARD::MENU
GET_PLAN      COMMAND::MENU    CARD::VIEW
GET_AM        COMMAND::MENU    CARD::VIEW
GET_PM        COMMAND::MENU    CARD::VIEW
GET_EVE       COMMAND::MENU    CARD::VIEW
GET_EDITPLAN  COMMAND::VIEW    CARD::EDIT
GET_EDITAM    COMMAND::VIEW    CARD::EDIT
GET_EDITPM    COMMAND::VIEW    CARD::EDIT
GET_EDITEVE   COMMAND::VIEW    CARD::EDIT
GET_BACK      COMMAND::VIEW    CARD::MENU
POST_EDITPLAN COMMAND::EDIT    CARD::MENU
POST_EDITAM   COMMAND::EDIT    CARD::MENU
POST_EDITPM   COMMAND::EDIT    CARD::MENU
POST_EDITEVE  COMMAND::EDIT    CARD::MENU
GET_CANCEL    COMMAND::EDIT    CARD::MENU
GET_LOGOUT    COMMAND::LOGOUT  CARD::LOGOUT
END

$ical = ICALENDAR->new;
CARD::ERROR->print unless($ical->parse);
$key     = $ical->{PROTO} . '_' . $ical->{TYPE};
$cName   = $table{$key}{COMMAND};
$card    = $table{$key}{CARD};

if(defined $cName && defined $card){
  $command = $cName->new($card);
  if($cName eq 'COMMAND::GETDECK'){
    do{
      $cooky = int(rand(999999999))+1;
    }while($ical->cookyIexist($cooky));
    $deck = new DECK($cooky);
    $ical->decksIpush($deck);
  }else{
    $deck = $ical->decksIsearch;
  }
  if($deck){
    $deck->exec($command,$ical);
  }else{
    CARD::ERROR->print;
  }
}else{
  CARD::ERROR->print;
}
$ical->exit;
__DATA__

=head1 

ical.pl i[hpJ_[c[

=head1 Tv

i[hpCGI̎pƂčJ_[c[łB
users.pluserIDƃpX[h̓t@C
ƂKvłBKvdecksƂt@C
[U̓͂f[^ێnotesƂt@C쐬
܂BR}hCNƃfobOp̃vvg
łĂ܂B̃t@Ccgi-binfBNgɁAindex.html
htdocst@CɂĂB

=head1 o[W

̃vÓ@ical.pl ver1.0ł

=head1 

Copyright (c) 2000, \ (keiichi@tokyo.pm.org)
Permission is hereby granted to distribute this document
intact with the Perl distribution, and in accordance with
the licenses of the Perl distribution; derived documents
must include this copyright notice intact.

=cut
